@@ -167,10 +167,17 @@ int wrapperGetLastError() {
  * filename: File to write to.
  * pid: pid to write in the file.
  */
-int writePidFile(const TCHAR *filename, DWORD pid, int newUmask) {
+int writePidFile(const TCHAR *filename, DWORD pid, int newUmask, int exclusive) {
     FILE *pid_fp = NULL;
     int old_umask;
 
+    if (getBooleanProperty(properties, TEXT("wrapper.pidfile.strict"), FALSE, FALSE) == TRUE && 
+        exclusive == TRUE && file_exists(filename)) {
+        log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
+            TEXT("%d pid file, %s, already exists."), pid, filename);
+        return 1;
+    }
+
     old_umask = umask(newUmask);
     pid_fp = _tfopen(filename, TEXT("w"));
     umask(old_umask);
@@ -412,11 +419,13 @@ const TCHAR* getSignalCodeDesc(int code) {
  * Describe a signal.  This is called from within signal handlers so NO MALLOCs are allowed here.
  */
 void descSignal(siginfo_t *sigInfo) {
+#ifdef SI_USER
     struct passwd *pw;
-#ifdef UNICODE
+ #ifdef UNICODE
     size_t req;
-#endif
+ #endif
     TCHAR *uName;
+#endif
 
     /* Not supported on all platforms */
     if (sigInfo == NULL) {
@@ -456,9 +465,9 @@ void descSignal(siginfo_t *sigInfo) {
             if (pw == NULL) {
                 uName = TEXT("<unknown>");
             } else {
-#ifndef UNICODE
+ #ifndef UNICODE
                 uName = pw->pw_name;
-#else
+ #else
                 req = mbstowcs(NULL, pw->pw_name, 0) + 1;
                 uName = malloc(req * sizeof(TCHAR));
                 if (!uName) {
@@ -466,20 +475,20 @@ void descSignal(siginfo_t *sigInfo) {
                     return;
                 }
                 mbstowcs(uName, pw->pw_name, req + 1);
-#endif
+ #endif
             }
 
             /* It appears that the getsid function was added in version 1.3.44 of the linux kernel. */
             log_printf_queue(TRUE, WRAPPER_SOURCE_WRAPPER, LEVEL_DEBUG,
-#ifdef UNICODE
+ #ifdef UNICODE
                 TEXT("  signal generated by PID: %d (Session PID: %d), UID: %d (%S)"),
-#else
+ #else
                 TEXT("  signal generated by PID: %d (Session PID: %d), UID: %d (%s)"),
-#endif
+ #endif
                 sigInfo->si_pid, getsid(sigInfo->si_pid), sigInfo->si_uid, uName);
-#ifdef UNICODE
+ #ifdef UNICODE
             free(uName);
-#endif
+ #endif
         }
 #endif
     }
@@ -1032,21 +1041,10 @@ int wrapperSleep(int ms) {
  * Detaches the Java process so the Wrapper will if effect forget about it.
  */
 void wrapperDetachJava() {
-    int fd;
-    
     wrapperSetJavaState(WRAPPER_JSTATE_DOWN_CLEAN, 0, -1);
     
-    /* Redirect the pipes with the JVM to /dev/null so the JVM doesn't block when it writes to them. */
-    fd = _topen(TEXT("/dev/null"), O_RDWR, 0);
-    if (fd != -1) {
-        close(pipedes[STDOUT_FILENO]);
-        /*dup2(fd, pipedes[STDOUT_FILENO]); */
-        close(pipedes[STDERR_FILENO]);
-        /* dup2(fd, pipedes[STDERR_FILENO]); */
-        /*if ((fd != pipedes[STDOUT_FILENO]) && (fd != pipedes[STDERR_FILENO])) { */
-        close(fd);
-        /*}*/
-    }
+    /* Leave the stdout/stderr pipe in pipedes alone so we grab any remaining output.
+     *  They should have been redirected on the JVM side anyway. */
 }
 
 
@@ -1262,7 +1260,7 @@ void wrapperExecute() {
 
             /* If a java pid filename is specified then write the pid of the java process. */
             if (wrapperData->javaPidFilename) {
-                if (writePidFile(wrapperData->javaPidFilename, wrapperData->javaPID, wrapperData->javaPidFileUmask)) {
+                if (writePidFile(wrapperData->javaPidFilename, wrapperData->javaPID, wrapperData->javaPidFileUmask, FALSE)) {
                     log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_WARN,
                         TEXT("Unable to write the Java PID file: %s"), wrapperData->javaPidFilename);
                 }
@@ -1270,7 +1268,7 @@ void wrapperExecute() {
 
             /* If a java id filename is specified then write the pid of the java process. */
             if (wrapperData->javaIdFilename) {
-                if (writePidFile(wrapperData->javaIdFilename, wrapperData->jvmRestarts, wrapperData->javaIdFileUmask)) {
+                if (writePidFile(wrapperData->javaIdFilename, wrapperData->jvmRestarts, wrapperData->javaIdFileUmask, FALSE)) {
                     log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_WARN,
                         TEXT("Unable to write the Java Id file: %s"), wrapperData->javaIdFilename);
                 }
@@ -1663,7 +1661,6 @@ int main(int argc, char **argv) {
     int i;
 #endif
     TCHAR *retLocale;
-    int localeSet;
 #ifdef UNICODE
     size_t req;
     TCHAR **argv;
@@ -1696,7 +1693,6 @@ int main(int argc, char **argv) {
 #if defined(UNICODE)
         free(retLocale);
 #endif
-        localeSet = TRUE;
     } else {
         /* TODO - We need to be careful about LANG here as it is not set on all systems. */
         /*
@@ -1706,7 +1702,6 @@ int main(int argc, char **argv) {
         free(envLang);
 #endif
         */
-        localeSet = FALSE;
     }
     if (wrapperInitialize()) {
         appExit(1, argc, argv);
@@ -1837,10 +1832,32 @@ int main(int argc, char **argv) {
             rollLogs();
         }
 
+        if (wrapperData->pidFilename) {
+            if (writePidFile(wrapperData->pidFilename, wrapperData->wrapperPID, wrapperData->pidFileUmask, TRUE)) {
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL,
+                     TEXT("ERROR: Could not write pid file %s: %s"),
+                wrapperData->pidFilename, getLastErrorText());
+                     /* Common wrapper cleanup code. */
+                wrapperDispose();
+#if defined(UNICODE)
+                for (i = 0; i < argc; i++) {
+                    if (argv[i]) {
+                        free(argv[i]);
+                    }
+                }
+                if (argv) {
+                    free(argv);
+                }
+#endif
+                exit(1);
+                return 1; /* For compiler. */
+            }
+        }
+
         /* Write pid and anchor files as requested.  If they are the same file the file is
          *  simply overwritten. */
         if (wrapperData->anchorFilename) {
-            if (writePidFile(wrapperData->anchorFilename, wrapperData->wrapperPID, wrapperData->anchorFileUmask)) {
+            if (writePidFile(wrapperData->anchorFilename, wrapperData->wrapperPID, wrapperData->anchorFileUmask, FALSE)) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL,
                      TEXT("ERROR: Could not write anchor file %s: %s"),
                      wrapperData->anchorFilename, getLastErrorText());
@@ -1848,17 +1865,9 @@ int main(int argc, char **argv) {
                 return 1; /* For compiler. */
             }
         }
-        if (wrapperData->pidFilename) {
-            if (writePidFile(wrapperData->pidFilename, wrapperData->wrapperPID, wrapperData->pidFileUmask)) {
-                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL,
-                     TEXT("ERROR: Could not write pid file %s: %s"),
-                     wrapperData->pidFilename, getLastErrorText());
-                appExit(1, argc, argv);
-                return 1; /* For compiler. */
-            }
-        }
+
         if (wrapperData->lockFilename) {
-            if (writePidFile(wrapperData->lockFilename, wrapperData->wrapperPID, wrapperData->lockFileUmask)) {
+            if (writePidFile(wrapperData->lockFilename, wrapperData->wrapperPID, wrapperData->lockFileUmask, FALSE)) {
                 /* This will fail if the user is running as a user without full privileges.
                  *  To make things easier for user configuration, this is ignored if sufficient
                  *  privileges do not exist. */
